require "rubygems"
require "eventmachine"
require "fiber"

class FastAgiServer < EventMachine::Connection
  attr_accessor :d
  attr_accessor :next_method
  attr_accessor :hanguped_up
  attr_accessor :fiber
  attr_accessor :vars
  
  
  def init
    self.d = ""
    self.next_method = "read_vars"
    self.hanguped_up = false
    self.vars = Hash.new
  end
  
  def post_init
    init
    puts "New Connection!"
  end

  def receive_data data
    self.send(self.next_method, data)
  end
  
  def unbind    
    puts "CLOSED"
  end
  
  def hangup?
    self.hanguped_up
  end
  
  def read_vars(data)
    self.d = "#{self.d}#{data}"
    if self.d.index("\n\n")
      self.next_method = "process_data"
      self.d.gsub("\n\n", "").split("\n").each do |v|
        m = v.match(/(.*): (.*)/)
        if m
          self.vars[m[1]] = m[2]
        end
      end
      Fiber.new {
        run
      }.resume
    end
  end
  
  def run
    puts "channel: #{self.vars['agi_channel']}"    
    answer
    execute("Playback", "beep&beep")
    puts hangup
  end
  
  
  def method_missing(sim, *args)
    self.fiber = Fiber.current
    self.send("cmd_#{sim}", args)
    return Fiber.yield        
  end
  
  def cmd_execute(args)
    send_data "exec #{args[0]} #{args[1]}\n"
  end
  
  def cmd_answer(args)
    send_data "answer\n"
  end
  
  def cmd_hangup(args)
    send_data "hangup\n"
  end
    
  
  def process_data(data)
    if data == "HANGUP\n"
      puts "Received hangup"
      self.hanguped_up = true
      if self.fiber.alive?
        self.fiber.resume data
      end
    else
      self.fiber.resume data
    end
  end
  
end


EventMachine::run {
  EventMachine::start_server "0.0.0.0", 9090, FastAgiServer
  puts 'Running FAGI Server on port 9090'
}
